@@ -3,7 +3,7 @@
 /**
  * DB based User-to-Email and Email-to-User lookup
  *
- * Add it to the plugins list in config/main.inc.php and set
+ * Add it to the plugins list in config.inc.php and set
  * SQL queries to resolve usernames, e-mail addresses and hostnames from the database
  * %u will be replaced with the current username for login.
  * %m will be replaced with the current e-mail address for login.
@@ -12,23 +12,29 @@
  * The email query could optionally select identity data columns in specified order:
  *    name, organization, reply-to, bcc, signature, html_signature
  *
- * $rcmail_config['virtuser_query'] = array('email' => '', 'user' => '', 'host' => '');
+ * $config['virtuser_query'] = array('email' => '', 'user' => '', 'host' => '', 'alias' => '');
  *
  * The email query can return more than one record to create more identities.
  * This requires identities_level option to be set to value less than 2.
  *
+ * By default Roundcube database is used. To use different database (or host)
+ * you can specify DSN string in $config['virtuser_query_dsn'] option.
+ *
  * @version @package_version@
  * @author Aleksander Machniak <alec@alec.pl>
  * @author Steffen Vogel
+ * @author Tim Gerundt
+ * @license GNU GPLv3+
  */
 class virtuser_query extends rcube_plugin
 {
     private $config;
     private $app;
+    private $db;
 
     function init()
     {
-        $this->app = rcmail::get_instance();
+        $this->app    = rcmail::get_instance();
         $this->config = $this->app->config->get('virtuser_query');
 
         if (!empty($this->config)) {
@@ -45,6 +51,9 @@ class virtuser_query extends rcube_plugin
             if ($this->config['host']) {
                 $this->add_hook('authenticate', array($this, 'user2host'));
             }
+            if ($this->config['alias']) {
+                $this->add_hook('authenticate', array($this, 'alias2user'));
+            }
         }
     }
 
@@ -53,7 +62,7 @@ class virtuser_query extends rcube_plugin
      */
     function user2email($p)
     {
-        $dbh = $this->app->get_dbh();
+        $dbh = $this->get_dbh();
 
         $sql_result = $dbh->query(preg_replace('/%u/', $dbh->escape($p['user']), $this->config['email']));
 
@@ -61,11 +70,11 @@ class virtuser_query extends rcube_plugin
             if (strpos($sql_arr[0], '@')) {
                 if ($p['extended'] && count($sql_arr) > 1) {
                     $result[] = array(
-                        'email'         => rcube_idn_to_ascii($sql_arr[0]),
+                        'email'         => rcube_utils::idn_to_ascii($sql_arr[0]),
                         'name'          => $sql_arr[1],
                         'organization'  => $sql_arr[2],
-                        'reply-to'      => rcube_idn_to_ascii($sql_arr[3]),
-                        'bcc'           => rcube_idn_to_ascii($sql_arr[4]),
+                        'reply-to'      => rcube_utils::idn_to_ascii($sql_arr[3]),
+                        'bcc'           => rcube_utils::idn_to_ascii($sql_arr[4]),
                         'signature'     => $sql_arr[5],
                         'html_signature' => (int)$sql_arr[6],
                     );
@@ -90,7 +99,7 @@ class virtuser_query extends rcube_plugin
      */
     function email2user($p)
     {
-        $dbh = $this->app->get_dbh();
+        $dbh = $this->get_dbh();
 
         $sql_result = $dbh->query(preg_replace('/%m/', $dbh->escape($p['email']), $this->config['user']));
 
@@ -106,7 +115,7 @@ class virtuser_query extends rcube_plugin
      */
     function user2host($p)
     {
-        $dbh = $this->app->get_dbh();
+        $dbh = $this->get_dbh();
 
         $sql_result = $dbh->query(preg_replace('/%u/', $dbh->escape($p['user']), $this->config['host']));
 
@@ -117,5 +126,40 @@ class virtuser_query extends rcube_plugin
         return $p;
     }
 
-}
+    /**
+     * Alias > User
+     */
+    function alias2user($p)
+    {
+        $dbh = $this->get_dbh();
 
+        $sql_result = $dbh->query(preg_replace('/%u/', $dbh->escape($p['user']), $this->config['alias']));
+
+        if ($sql_arr = $dbh->fetch_array($sql_result)) {
+            $p['user'] = $sql_arr[0];
+        }
+
+        return $p;
+    }
+
+    /**
+     * Initialize database handler
+     */
+    function get_dbh()
+    {
+        if (!$this->db) {
+            if ($dsn = $this->app->config->get('virtuser_query_dsn')) {
+                // connect to the virtuser database
+                $this->db = rcube_db::factory($dsn);
+                $this->db->set_debug((bool)$this->app->config->get('sql_debug'));
+                $this->db->db_connect('r'); // connect in read mode
+            }
+            else {
+                $this->db = $this->app->get_dbh();
+            }
+        }
+
+        return $this->db;
+    }
+
+}
